/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/





#include "conf.h"
#include "lists.h"
#include "main.h"

#include <stdio.h>

static
gboolean _file_write_line (FILE *f, const gchar *s);

void conf_read_profiles (void)
{
  xmlDocPtr doc;
  xmlNodePtr cur, pr;
  gchar *filename;
  xmlChar *key;
  gint active;
  Profile *p;
  gchar *u;

  tlen_debug ("read profiles\n");
  filename = g_strdup_printf ("%s/config.xml", config_dir);
  doc = xmlParseFile (filename);
  if (doc == NULL )
  {
    tlen_debug ("Error while loading profiles.\n");
    return;
  }
  cur = xmlDocGetRootElement (doc);
  while (cur)
  {
    if (!xmlStrcmp(cur->name, (const xmlChar *) "profiles"))
    {
      cur = cur->xmlChildrenNode;
      continue;
    }
    if (!xmlStrcmp(cur->name, (const xmlChar *) "profile"))
    {
      p = list_profile_item_new ();
      pr = cur->xmlChildrenNode;
      if (p)
      while (pr)
      {
        if (!xmlStrcmp(pr->name, (const xmlChar *) "name"))
        {
          u = (gchar*) xmlNodeListGetString(doc, pr->xmlChildrenNode, 1);
          p->name = tlen_decode (u);
          g_free (u);
        }
        else
        if (!xmlStrcmp(pr->name, (const xmlChar *) "tlen_login"))
        {
          u = (gchar*) xmlNodeListGetString(doc, pr->xmlChildrenNode, 1);
          p->tlen_login = tlen_decode (u);
          g_free (u);
        }
        else
        if (!xmlStrcmp(pr->name, (const xmlChar *) "tlen_password"))
        {
          u = (gchar*) xmlNodeListGetString(doc, pr->xmlChildrenNode, 1);
          p->tlen_password = tlen_decode (u);
          g_free (u);
        }
        else
        if (!xmlStrcmp(pr->name, (const xmlChar *) "active"))
        {
          key = (gchar*) xmlNodeListGetString(doc, pr->xmlChildrenNode, 1);
          if (key)
          {
            active = atoi ((gchar*) key);
            if (active)
              p->active = TRUE;
            xmlFree (key);
          }
        }
        pr = pr->next;
      }
      list_profile_item_add (p);
    }
    cur = cur->next;
  }
  xmlFreeDoc (doc);
  g_free (filename);
}

void conf_save_profiles ()
{
  GSList *l = list_profiles;
  Profile *p;
  gchar *filename, *s;
  FILE *f;

  filename = g_strdup_printf ("%s/config.xml", config_dir);
  f = fopen (filename, "w");
  g_free (filename);
  if (!f)
  {
    tlen_debug ("Cannot create file in %s\n", config_dir);
    perror (config_dir);
    return;
  }
  if (!_file_write_line (f, "<profiles>\n"))
  {
    fclose (f);
    return;
  }
  while (l)
  {
    p = (Profile *) l->data;
    s = profile_get_xml (p);
    if (s)
    {
      if (!_file_write_line (f, s))
      {
        fclose (f);
        g_free (s);
        return;
      }
      g_free (s);
    }
    l = l->next;
  }
  if (!_file_write_line (f, "</profiles>\n"))
  {
    fclose (f);
    return;
  }
  fclose (f);
}

static
gboolean _file_write_line (FILE *f, const gchar *s)
{
  gint len;

  if (!f || !s)
    return FALSE;
  len = tlen_strlen (s);
  if (fwrite (s, len, 1, f) != 1)
  {
    tlen_debug ("Error writing to file\n");
    return FALSE;
  }
  return TRUE;
}
