/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/





// list management, structures, memory allocation, frees, etc.

#include "lists.h"
#include "utils.h"
#include <libtlen2/libtlen2.h>
#include <string.h>


// single linked lists
GSList
  *list_users,
  *list_talks, //list of opened talk windows
  *list_messages, //list of normal messages
  *list_p2p, //list of p2p windows
  *list_profiles;


gchar *opisy[7]={"Niedostpny", "Niewidoczny", "Wrc pniej", "Zaraz wracam",
  "Jestem zajty", "Porozmawiajmy" , "Dostpny"};


void list_free ()
{
  list_user_free ();
  list_profile_free ();
  list_talk_free ();
  list_message_free ();
  list_p2p_free ();
}

static
GSList *_list_user_item_find (GSList *l, const gchar *jid);
static
GSList *_list_profile_item_find (GSList *l, const gchar *name);
static
gint _profile_compare_func (gconstpointer a, gconstpointer b);
static
void _profile_clear_active (void);
static
gint _list_user_compare_func(gconstpointer a, gconstpointer b);


P2p *list_p2p_item_new ()
{
  P2p *p;

  p = g_new0 (P2p, 1);
  return p;
}

void list_p2p_item_add (P2p *p)
{
  list_p2p = g_slist_append (list_p2p, p);
}

P2p *list_p2p_item_get_by_window (GtkWidget *window)
{
  GSList *l;
  P2p *u = NULL;

  l = list_p2p;
  while (l)
  {
    u = (P2p*) l->data;
    if (u && (u->window == window))
      break;
    u = NULL;
    l = l->next;
  }
  return u;
}

void list_p2p_item_del (P2p *p)
{
  if (p)
  {
    list_p2p = g_slist_remove (list_p2p, p);
    list_p2p_item_free (p);
  }
}

void list_p2p_item_free (P2p *p)
{
  if (p)
  {
    g_free (p);
  }
}

void list_p2p_free ()
{
  GSList *l;
  P2p *u;

  l = list_p2p;
  while (l)
  {
    u = (P2p*) l->data;
    list_p2p_item_free (u);
    l = l->next;
  }
  g_slist_free (list_p2p);
  list_p2p = NULL;
}

void list_message_free (void)
{
  GSList *l;
  Message *u;

  l = list_messages;
  while (l)
  {
    u = (Message*) l->data;
    list_message_item_free (u);
    l = l->next;
  }
  g_slist_free (list_messages);
  list_messages = NULL;
}

void list_message_item_free (Message *m)
{
  if (m)
  {
    g_free (m->jid);
    g_free (m->body);
    g_free (m->stamp);
    g_free (m);
  }
}

void list_message_item_add (Message *m)
{
  if (m)
    list_messages = g_slist_append (list_messages, (gpointer) m);
}

Message *list_message_item_new (const gchar *jid, const gchar *body,
                                const gchar *stamp)
{
  Message *m;

  m = g_new0 (Message, 1);
  if (m)
  {
    m->jid = g_strdup (jid);
    m->body = g_strdup (body);
    m->stamp = g_strdup (stamp);
    m->read = FALSE;
  }
  return m;
}

Talk *list_talk_item_get_by_window (GtkWidget *window)
{
  GSList *l;
  Talk *u = NULL;

  l = list_talks;
  while (l)
  {
    u = (Talk*) l->data;
    if (u && (u->window == window))
      break;
    u = NULL;
    l = l->next;
  }
  return u;
}

Talk *list_talk_item_get (const gchar *jid)
{
  GSList *l;
  Talk *u = NULL;

  l = list_talks;
  while (l)
  {
    u = (Talk*) l->data;
    if (u && !tlen_strcmp (u->jid, jid))
      break;
    u = NULL;
    l = l->next;
  }
  return u;
}

void list_talk_free ()
{
  GSList *l;
  Talk *u;

  l = list_talks;
  while (l)
  {
    u = (Talk*) l->data;
    list_talk_item_free (u);
    l = l->next;
  }
  g_slist_free (list_talks);
  list_talks = NULL;
}

void list_talk_item_add (Talk *t)
{
  if (t)
    list_talks = g_slist_append (list_talks, (gpointer) t);
}

void list_talk_item_free (Talk *t)
{
  if (t)
  {
    g_free (t->jid);
    g_free (t);
  }
}

Talk *list_talk_item_new (GtkWidget *window, const gchar *jid)
{
  Talk *t;

  t = g_new0 (Talk, 1);
  if (t)
  {
    t->window = window;
    t->jid = g_strdup (jid);
  }
  return t;
}

gchar *profile_get_xml (const Profile *p)
{
  gchar *s = NULL;
  gchar *name, *tlen_login, *tlen_password;

  if (p)
  {
    name = tlen_encode (p->name);
    tlen_login = tlen_encode (p->tlen_login);
    tlen_password = tlen_encode (p->tlen_password);
    s = g_strdup_printf
            ("<profile>\n<name>%s</name>\n<active>%d</active>\n<tlen_login>%s</tlen_login>\n<tlen_password>%s</tlen_password>\n</profile>\n",
            name, p->active, tlen_login, tlen_password);
    g_free (name);
    g_free (tlen_login);
    g_free (tlen_password);
  }
  return s;
}

Profile *list_profile_item_new ()
{
  Profile *u;

  u = g_new0 (Profile, 1);
  return u;
}

void list_profile_item_free (Profile *u)
{
  if (u)
  {
    g_free (u->name);
    g_free (u->tlen_login);
    g_free (u->tlen_password);
    g_free (u);
  }
}

void list_profile_item_add (Profile *u)
{
  Profile *p;

  if (u)
  {
    p = list_profile_item_get (u->name);
    if (p)
      list_profile_item_del (p);
    if (u->active)
      _profile_clear_active ();
    list_profiles = g_slist_insert_sorted (list_profiles, u,
                                           _profile_compare_func);
  }
}

void list_profile_item_del (Profile *u)
{
  GSList *l;

  if (!u)
    return;
  l = _list_profile_item_find (list_profiles, u->name);
  if (l)
  {
    list_profiles = g_slist_remove (list_profiles, l->data);
    list_profile_item_free ((Profile*) l->data);
  }
}

void list_profile_free ()
{
  GSList *l;
  Profile *u;

  l = list_profiles;
  while (l)
  {
    u = (Profile*) l->data;
    list_profile_item_free (u);
    l = l->next;
  }
  g_slist_free (list_profiles);
  list_profiles = NULL;
}

Profile *list_profile_item_get (const gchar *name)
{
  GSList *l;

  l = _list_profile_item_find (list_profiles, name);
  if (l)
    return (Profile*) l->data;
  return NULL;
}

static
GSList *_list_profile_item_find (GSList *l, const gchar *name)
{
  Profile *u;

  if (!name)
    return NULL;
  while (l)
  {
    u = (Profile*) l->data;
    if (!tlen_strcmp (u->name, name))
      return l;
    l = l->next;
  }
  return NULL;
}

User *list_user_item_new ()
{
  User *u;

  u = g_new0 (User, 1);
  return u;
}

void list_user_item_free (User *u)
{
  if (u)
  {
    g_free (u->jid);
    g_free (u->name);
    g_free (u->group);
    g_free (u->description);
    g_free (u->subscription);
    g_free (u);
  }
}

void list_user_item_append (User *u)
{
  User *user;

  if (!u)
    return;
  user = list_user_item_get (u->jid);
  if (!user)
    list_users = g_slist_append (list_users, u);
  else
  {
    if (tlen_strcmp (u->name, user->name))
    {
      if (u->name)
      {
        g_free (user->name);
        user->name = g_strdup (u->name);
      }
    }
    if (tlen_strcmp (u->ask, user->ask))
    {
      if (u->ask)
      {
        g_free (user->ask);
        user->ask = g_strdup (u->ask);
      }
    }
    if (tlen_strcmp (u->subscription, user->subscription))
    {
      if (u->subscription)
      {
        g_free (user->subscription);
        user->subscription= g_strdup (u->subscription);
      }
    }
    if (tlen_strcmp (u->group, user->group))
    {
      if (u->group)
      {
        g_free (user->group);
        user->group = g_strdup (u->group);
      }
    }
    list_user_item_free (u);
  }
}

void list_user_item_update (User *u, UserDataType dtype, gpointer s)
{
  if (!u)
    return;
  switch (dtype)
  {
    case USER_DATA_DESCRIPTION:
      g_free (u->description);
      u->description  = g_strdup ((gchar*) s);
      break;
    default:
      break;
  }
}

void list_user_item_del (User *u)
{
  GSList *l;

  if (!u)
    return;
  l = _list_user_item_find (list_users, u->jid);
  if (l)
  {
    list_users = g_slist_remove (list_users, l->data);
    list_user_item_free ((User*) l->data);
  }
}

void list_user_free ()
{
  GSList *l;
  User *u;

  l = list_users;
  while (l)
  {
    u = (User*) l->data;
    list_user_item_free (u);
    l = l->next;
  }
  g_slist_free (list_users);
  list_users = NULL;
}

User *list_user_item_get (const gchar *jid)
{
  GSList *l;
  gchar *p;

  if (!strchr (jid, '@'))
    p = g_strdup_printf ("%s@tlen.pl", jid);
  else
    p = g_strdup (jid);
  l = _list_user_item_find (list_users, p);
  g_free (p);
  if (l)
    return (User*) l->data;
  return NULL;
}

gchar *list_user_get_name (const gchar *jid)
{
  User *u;
  gchar *s, *ret;

  if (!jid)
    return NULL;
  u = list_user_item_get (jid);
  if (!u)
    return g_strdup (jid);
  if (tlen_strlen (u->name))
    return g_strdup (u->name);
  s = strchr (jid, '@');
  if (s)
    *s = '\0';
  ret = g_strdup (jid);
  if (s)
    *s = '@';
  return ret;
}

gchar *user_description (User *user)
{
  gchar *s = NULL;

  if (tlen_strlen (user->description))
  {
    if (tlen_strcmp (user->description, opisy[user->status]))
    {
      if (user->type == USER_TYPE_TLEN)
        s = utf (user->description);
      else
        s = g_strdup (user->description);
    }
  }
  return s;
}

gboolean user_subscribed (User *u)
{
  if (!u)
    return FALSE;
  if (!tlen_strcmp (u->subscription, "both") ||
      !tlen_strcmp (u->subscription, "to"))
    return TRUE;
  return FALSE;
}

void list_user_sort (void)
{
  list_users = g_slist_sort (list_users, _list_user_compare_func);
}

static
gint _list_user_compare_func (gconstpointer a, gconstpointer b)
{
  User *u1, *u2;
  guint status1, status2;
  gchar *user1, *user2;
  gint ret;

  u1 = (User*) a;
  u2 = (User*) b;
  status1 = u1->status;
  status2 = u2->status;
  if (status1 < status2)
    return 1;

  user1 = list_user_get_name (u1->jid);
  user2 = list_user_get_name (u2->jid);
  if (status1 == status2)
    ret = strcasecmp (user1, user2);
  else
    ret = -1;
  g_free (user1);
  g_free (user2);
  return ret;
}

static
GSList *_list_user_item_find (GSList *l, const gchar *jid)
{
  User *u;

  if (!jid)
    return NULL;
  while (l)
  {
    u = (User*) l->data;
    if (!tlen_strcmp (u->jid, jid))
      return l;
    l = l->next;
  }
  return NULL;
}

static
gint _profile_compare_func (gconstpointer a, gconstpointer b)
{
  Profile *p, *r;

  p = (Profile*) a;
  r = (Profile*) b;
  if (p && r)
  {
    return tlen_strcmp (p->name, r->name);
  }
  return 0;
}

static
void _profile_clear_active (void)
{
  GSList *s;
  Profile *p;

  s = list_profiles;
  while (s)
  {
    p = (Profile*) s->data;
    p->active = FALSE;
    s = s->next;
  }
}

void frees (gpointer d1, gpointer d2)
{
  g_free ((gchar*) d1);
}

