/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/




#ifndef __LISTS_H__
#define __LISTS_H__


#include <gtk/gtk.h>
#include <libtlen2/libtlen2.h>

extern
GSList
  *list_users,
  *list_talks,
  *list_messages,
  *list_p2p,
  *list_profiles;

extern gchar
  *opisy[];

typedef enum {
  USER_STATUS_UNAVAILABLE = 0,
  USER_STATUS_INVISIBLE,
  USER_STATUS_EXT_AWAY,
  USER_STATUS_AWAY,
  USER_STATUS_DND,
  USER_STATUS_CHATTY,
  USER_STATUS_AVAILABLE
} UserStatus;

typedef enum {
  USER_DATA_DESCRIPTION
} UserDataType;

typedef enum {
  USER_TYPE_TLEN = 0
} UserType;

typedef struct
{
  GtkWidget *window;
  GtkWidget *fileselection;
} P2p;

typedef struct
{
  gchar *jid;
  gchar *name;
  gchar *group;
  gchar *subscription;
  gchar *description;
  gchar *ask;
  UserStatus status;
  UserType type;
} User;

typedef enum {
  USER_STATE_QUIT = 0,
  USER_STATE_CHANGE_PROFILE,
} UserState;

typedef struct
{
  gchar *name;
  gchar *tlen_login;
  gchar *tlen_password;
  gboolean active;
  TlenConnection *tc;
  UserStatus status;
  UserState state;
} Profile;

typedef struct
{
  GtkWidget *window;
  gchar *jid;
} Talk;

typedef struct
{
  gchar *jid;
  gchar *body;
  gchar *stamp;
  gboolean read;
} Message;


P2p *list_p2p_item_new ();
void list_p2p_item_add (P2p *p);
P2p *list_p2p_item_get_by_window (GtkWidget *window);
void list_p2p_item_free (P2p *p);
void list_p2p_item_del (P2p *p);
void list_p2p_free ();
User *list_user_item_new ();
void list_user_item_free (User *u);
void list_user_item_append (User *u);
void list_user_item_del (User *u);
User *list_user_item_get (const gchar *jid);
gchar *list_user_get_name (const gchar *jid);
void list_user_free ();
void list_user_sort (void);
void list_free ();

Profile *list_profile_item_new ();
void list_profile_item_free (Profile *u);
void list_profile_item_add (Profile *u);
void list_profile_item_del (Profile *u);
void list_profile_free ();
Profile *list_profile_item_get (const gchar *name);
gchar *profile_get_xml (const Profile *p);
Talk *list_talk_item_new (GtkWidget *window, const gchar *jid);
void list_talk_item_free (Talk *t);
void list_talk_item_add (Talk *t);
Talk *list_talk_item_get (const gchar *jid);
void list_user_item_update (User *u, UserDataType dtype, gpointer s);
Talk *list_talk_item_get_by_window (GtkWidget *window);
void list_talk_free ();
Message *list_message_item_new (const gchar *jid, const gchar *body,
                                const gchar *stamp);
void list_message_item_add (Message *m);
void list_message_item_free (Message *m);
void list_message_free (void);
gboolean user_subscribed (User *u);
gchar *user_description (User *user);

void frees (gpointer d1, gpointer d2);

#endif

