/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/





#include "tlen-events.h"
#include "lists.h"
#include "window-main.h"
#include "window-chat.h"
#include "window-login.h"
#include "window-misc.h"
#include "window-tuba.h"
#include "event-message.h"
#include "main.h"
#include "utils.h"

#include <string.h>

static
void _on_subscription_request (const gchar *id);
static
void _on_unsubscribed (const gchar *id);
static
void _on_unsubscribe (const gchar *id);
static
void _subscription_accept (gpointer d1, gpointer d2);
static
void _subscription_deny (gpointer d1, gpointer d2);

void tlen_event_cb (TlenConnection *tc)
{
  TlenEvent *ev;
  GSList *l;
  static gboolean init_roster = TRUE;

  if (tc)
  {
    while ((ev = tlen_event_get (tc)))
    {
      switch (ev->type)
      {
        case TLEN_EVENT_PRESENCE:
          {
            User *u;
          tlen_debug ("Event presence from: '%s', status: %d type: %s, description: '%s'\n",
                  TLEN_EVENT_PRESENCE (ev)->from,
                  TLEN_EVENT_PRESENCE (ev)->status,
                  TLEN_EVENT_PRESENCE (ev)->type,
                  TLEN_EVENT_PRESENCE (ev)->description);
          switch (tlen_presence_type (ev))
          {
            case TLEN_PRESENCE_SUBSCRIBED:
              break;
            case TLEN_PRESENCE_SUBSCRIBE:
              _on_subscription_request (TLEN_EVENT_PRESENCE (ev)->from);
              break;
            case TLEN_PRESENCE_UNSUBSCRIBE:
              _on_unsubscribe (TLEN_EVENT_PRESENCE (ev)->from);
              break;
            case TLEN_PRESENCE_UNSUBSCRIBED:
              _on_unsubscribed (TLEN_EVENT_PRESENCE (ev)->from);
              break;
            case TLEN_PRESENCE_STATUS:
              u = list_user_item_get (TLEN_EVENT_PRESENCE(ev)->from);
              if (u)
              {
                u->status = status_tlen2user (TLEN_EVENT_PRESENCE (ev)->status);
                list_user_item_update (u, USER_DATA_DESCRIPTION,
                                       TLEN_EVENT_PRESENCE(ev)->description);
                window_main_treeview_update ();
                window_chat_set_status (u->jid, u->status);
              }
              break;
          }
          break;
          }
        case TLEN_EVENT_CONNECTED:
          tlen_debug ("Event connected\n");
//          g_main_loop_quit (mainloop);
          break;
        case TLEN_EVENT_CONNECTION_FAILED:
          tlen_debug ("Event connection failed\n");
          window_main_close ();
          my_create_window_login ();
          info ("Wystpi bd podczas poczenia z serwerem.\nSprbuj ponownie.");
          break;
        case TLEN_EVENT_DISCONNECTED:
          tlen_debug ("Event disconnected\n");
          init_roster = TRUE;
          switch (profile->state)
          {
            case USER_STATE_QUIT:
              main_quit ();
              break;
            case USER_STATE_CHANGE_PROFILE:
              window_main_close ();
              my_create_window_login ();
              break;
          }
          break;
        case TLEN_EVENT_UNAUTHORIZED:
          tlen_debug ("Unauthorized event.\n");
          window_main_close ();
          my_create_window_login ();
          info ("Wystpi bd podczas logowania do serwera.\nSprawd haso.");
//          gtk_main_quit();
          break;
        case TLEN_EVENT_LOGGED:
          tlen_debug ("Logged in!\n");
          tlen_roster_get (tc);
//          g_main_loop_quit(mainloop);
          break;
        case TLEN_EVENT_SEARCH:
          tlen_debug ("Search event.\n");
          l = TLEN_EVENT_SEARCH (ev)->list;
          while (l)
          {
            TlenPubdirItem *item, *pubdir;

            item = (TlenPubdirItem*) l->data;
            if (item)
            {
              pubdir = tlen_pubdir_item_new();
              memcpy((void*)pubdir, (void*)item, sizeof(*pubdir));
              pubdir->firstname = g_strdup(item->firstname);
              pubdir->lastname = g_strdup(item->lastname);
              pubdir->nick = g_strdup(item->nick);
              pubdir->city = g_strdup(item->city);
              pubdir->email = g_strdup(item->email);
              pubdir->school = g_strdup(item->school);
              pubdir->id = g_strdup(item->id);
              pubdir_list_add(pubdir);
            }
            l = l->next;
          }
          if(window_search)
            window_search_update();
          else
            pubdir_list_clear ();

          break;
        case TLEN_EVENT_ROSTER:
          tlen_debug ("Roster event.\n");
          l = TLEN_EVENT_ROSTER (ev)->list;
          while (l)
          {
            User *u;
            gchar *p;
            TlenRosterItem *item;

            item = (TlenRosterItem*) l->data;
            if (item)
            {
              tlen_debug ("%s : %s : %s : %s\n", item->jid, item->name,
                      item->group, item->subscription);
              if (tlen_strcmp (item->subscription, "remove"))
              {
                u = list_user_item_new();
                u->jid = g_strdup (item->jid);
                p = item->name;
                u->name = g_strdup (p?p:"");
                p = item->group;
                u->group = g_strdup (p?p:"");
                p = item->ask;
                u->ask = g_strdup (p?p:"");
                p = item->subscription;
                u->subscription = g_strdup (p?p:"");
                list_user_item_append (u);
              }
              else
              {
                u = list_user_item_get (item->jid);
                if (u)
                  list_user_item_del (u);
              }
            }
            l = l->next;
          }
          if (init_roster)
            tlen_presence (tc, TLEN_PRESENCE_AVAILABLE, "");
          init_roster = FALSE;
          window_main_treeview_update ();
          profile->status = USER_STATUS_AVAILABLE;
          window_main_set_status (profile->status);
          break;
        case TLEN_EVENT_FILE:
          tlen_debug ("event file\n");
          switch (TLEN_EVENT_FILE (ev)->error)
          {
            case TLEN_P2P_ASK:
              tlen_debug ("user want to send a file\n");
              break;
            case TLEN_P2P_DENIED:
              tlen_debug ("file denied\n");
              break;
            case TLEN_P2P_ACCEPTED:
              tlen_debug ("file accepted\n");
              break;
            case TLEN_P2P_NOUSER:
              tlen_debug ("no such user\n");
              break;
            default:
              break;
          }
          break;
        case TLEN_EVENT_WEBMSG:
            my_create_window_webmsg (TLEN_EVENT_WEBMSG (ev)->from,
                                      TLEN_EVENT_WEBMSG (ev)->e,
                                      TLEN_EVENT_WEBMSG (ev)->body);
            break;
        case TLEN_EVENT_NEWMAIL:
            my_create_window_newmail (TLEN_EVENT_NEWMAIL (ev)->from,
                                      TLEN_EVENT_NEWMAIL (ev)->e);
            break;
        case TLEN_EVENT_MESSAGE:
          {
            gchar *stamp, *s = NULL;

            printf ("Event message from: '%s', type: %d body: '%s', stamp: '%s'\n",
                    TLEN_EVENT_MESSAGE (ev)->from,
                    TLEN_EVENT_MESSAGE (ev)->type,
                    TLEN_EVENT_MESSAGE (ev)->body,
                    TLEN_EVENT_MESSAGE (ev)->stamp);
            stamp = TLEN_EVENT_MESSAGE (ev)->stamp;
            if(stamp)
            {
              s = strchr (stamp, 'T');
              if (s)
                *s = ' ';
            }
            event_message_received (TLEN_EVENT_MESSAGE (ev)->from,
                                    TLEN_EVENT_MESSAGE (ev)->type,
                                    TLEN_EVENT_MESSAGE (ev)->body,
                                    stamp);
          }
          break;
        case TLEN_EVENT_PUBDIR:
          {
            TlenPubdirItem *item = TLEN_EVENT_PUBDIR (ev)->item;

            tlen_debug ("Pubdir event:\n");
            tlen_debug ("first: %s\n", item->firstname);
            tlen_debug ("last: %s\n", item->lastname);
            tlen_debug ("nick: %s\n", item->nick);
            tlen_debug ("email: %s\n", item->email);
            tlen_debug ("city: %s\n", item->city);
            tlen_debug ("school: %s\n", item->school);
            tlen_debug ("year: %d\n", item->birthyear);
            window_card_update (item);
          }
          break;
        case TLEN_EVENT_PUBDIR_CHANGED:
            tlen_debug ("Pubdir changed event:\n");
            window_card_hide_thobber ();
          break;

        default:
          tlen_debug ("event not recognized!\n");
          break;
      }
      tlen_event_free (ev);
    }
  }
}

static
void _on_unsubscribe (const gchar *id)
{
  User *u;

  tlen_unsubscribe_accept (profile->tc, id);
  u = list_user_item_get (id);
  if (u)
  {
    u->status = USER_STATUS_UNAVAILABLE;//FIXME
  }
  window_main_treeview_update ();
}

static
void _on_unsubscribed (const gchar *id)
{
  User *u;

  u = list_user_item_get (id);
  if (u)
  {
    u->status = USER_STATUS_UNAVAILABLE;//FIXME
  }
  window_main_treeview_update ();
}

static
void _on_subscription_request (const gchar *id)
{
  char *s;
  User *u;

  u = list_user_item_get (id);
  if (!u)
  {
    u = list_user_item_new ();
    u->jid = g_strdup (id);
    list_user_item_append (u);
    u = list_user_item_get (id);
  }
  tlen_subscribe_accept (profile->tc, id);
  if (!user_subscribed (u))
  {
    s = g_strdup_printf ("%s prosi Ci o autoryzacj.\nCzy chcesz go doda do swojej listy?",
                         id);
    //  tlen_debug ("%s\n", s);
    my_create_window_yesno (s, _subscription_accept, _subscription_deny,
                            (gpointer) u->jid, NULL);
    g_free (s);
  }
}

static
void _subscription_deny (gpointer d1, gpointer d2)
{
  User *u;

  tlen_debug ("here\n");
  u = list_user_item_get ((gchar*) d1);
  if (u)
  {
    tlen_contact_remove (profile->tc, (gchar*) d1);
    tlen_unsubscribe_request (profile->tc, (gchar*) d1);
    list_user_item_del (u);
    window_main_treeview_update ();
  }
}

static
void _subscription_accept (gpointer d1, gpointer d2)
{
  tlen_debug ("accepting subscription for %s\n", (gchar*) d1);
  my_create_window_contact ((gchar*) d1, 0);
}

