/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/



#include <gtk/gtk.h>
#include <time.h>
#include <libtlen2/libtlen2.h>

#include "utils.h"
#include "main.h"
#include "support.h"
#include "interface.h"
#include "window-main.h"



void status_button_set (UserStatus _status)
{
  GtkWidget *button;
  GList *widgets;
  GtkWidget *image;
  gchar *s, *ut;
  TlenPresenceStatus status;

  status = status_user2tlen (_status);
  button = lookup_widget (window_main, "button_main_status");
  widgets=gtk_container_get_children(GTK_CONTAINER(button));
  if(widgets)
  {
    gtk_container_remove(GTK_CONTAINER(button),(GtkWidget*)widgets->data);
    g_list_free(widgets);
  }
  switch(status)
  {
    case TLEN_PRESENCE_UNAVAILABLE:
      image = lookup_widget(window_main,"im_offline");
      break;
    case TLEN_PRESENCE_AWAY:
      image = lookup_widget(window_main,"im_away");
      break;
    case TLEN_PRESENCE_CHATTY:
      image = lookup_widget(window_main,"im_chat");
      break;
    case TLEN_PRESENCE_INVISIBLE:
      image = lookup_widget(window_main,"im_invisible");
      break;
    case TLEN_PRESENCE_EXT_AWAY:
      image = lookup_widget(window_main,"im_beback");
      break;
    case TLEN_PRESENCE_DND:
      image = lookup_widget(window_main,"im_occupied");
      break;
    case TLEN_PRESENCE_AVAILABLE:
    default:
      image = lookup_widget(window_main,"im_online");
  }
  gtk_container_add (GTK_CONTAINER (button), image);
  /* Set tooltip for Tlen.pl status icon */
  s = g_strdup_printf("Status Tlen: %s",opisy[_status]);
  ut = utf(s);
  g_free (s);
  gtk_tooltips_set_tip (status_tooltips, button, ut, NULL);
  g_free(ut);
}

void set_status (UserStatus status)
{
  if (!tlen_presence (profile->tc, status_user2tlen (status), NULL))
    info ("Wystpi bd poczenia");
  window_main_set_status (status);
}

TlenPresenceStatus status_user2tlen (UserStatus status)
{
  switch (status)
  {
    case USER_STATUS_AVAILABLE:
      return TLEN_PRESENCE_AVAILABLE;
      break;
    case USER_STATUS_UNAVAILABLE:
      return TLEN_PRESENCE_UNAVAILABLE;
      break;
    case USER_STATUS_AWAY:
    return TLEN_PRESENCE_AWAY;
      break;
    case USER_STATUS_DND:
      return TLEN_PRESENCE_DND;
      break;
    case USER_STATUS_EXT_AWAY:
    return TLEN_PRESENCE_EXT_AWAY;
      break;
    case USER_STATUS_CHATTY:
    return TLEN_PRESENCE_CHATTY;
      break;
    case USER_STATUS_INVISIBLE:
    return TLEN_PRESENCE_INVISIBLE;
      break;
  }
  return TLEN_PRESENCE_UNAVAILABLE;
}

UserStatus status_tlen2user (TlenPresenceStatus status)
{
  switch (status)
  {
    case TLEN_PRESENCE_AVAILABLE:
      return USER_STATUS_AVAILABLE;
      break;
    case TLEN_PRESENCE_UNAVAILABLE:
      return USER_STATUS_UNAVAILABLE;
      break;
    case TLEN_PRESENCE_AWAY:
      return USER_STATUS_AWAY;
      break;
    case TLEN_PRESENCE_DND:
      return USER_STATUS_DND;
      break;
    case TLEN_PRESENCE_EXT_AWAY:
      return USER_STATUS_EXT_AWAY;
      break;
    case TLEN_PRESENCE_CHATTY:
      return USER_STATUS_CHATTY;
      break;
    case TLEN_PRESENCE_INVISIBLE:
      return USER_STATUS_INVISIBLE;
      break;
  }
  return USER_STATUS_UNAVAILABLE;
}

gchar *get_localtime (const time_t *t2)
{
  struct tm *tm;
  time_t t;
  gchar s[9],s1[11], *s2;

  t = time (NULL);
  if (!t2)
    tm = localtime (&t);
  else
    tm = localtime (t2);
  strftime (s, 9, "%Y%m%d", tm);
  strftime (s1, 11, "%H:%M:%S", tm);
  if (t2)
    s2 = g_strdup_printf ("%s %s", s, s1);
  else
    s2 = g_strdup_printf ("%s", s1);
  return s2;
}

void textview_append_message (GtkWidget *textview, const gchar *jid,
                              const gchar *body, const gchar *stamp,
                              gboolean from)
{
  gchar *name, *u;

  name = list_user_get_name (jid);
  u = utf (name);
  g_free (name);
  if (!from)
    textview_append (textview, u, 0, "nickfg", NULL);
  else
    textview_append (textview, u, 0, "nickfg", "fontbold");
  textview_append (textview, " :: ", 0, NULL, NULL);
  g_free (u);
  if (stamp)
    u = g_strdup (stamp);
  else
    u = get_localtime (NULL);
  if (!from)
    textview_append(textview, u, 0, "datefg", NULL);
  else
    textview_append (textview, u, 0, "datefg", "fontbold");
  g_free (u);
  u = utf (body);
  textview_append (textview, "\n", 0, NULL, NULL);
  textview_append (textview, u, 0, NULL, NULL);
  textview_append (textview, "\n", 0, NULL, NULL);
  textview_append (textview, "\n", 1, NULL, NULL);
  g_free (u);
}

void textview_clear (GtkWidget *textview)
{
  GtkTextBuffer *buffer;
  GtkTextIter start, end;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
  gtk_text_buffer_get_start_iter (buffer, &start);
  gtk_text_buffer_get_end_iter (buffer, &end);
  gtk_text_buffer_delete (buffer, &start, &end);
}

void textview_append (GtkWidget *textview, const gchar *str, gint scroll,
                      const gchar *tag, const gchar *tag2)
{
  GtkTextBuffer *buffer;
  GtkTextIter end;
  GtkTextMark *mark;

  buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
  gtk_text_buffer_get_end_iter (buffer, &end);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &end,
                                            str, -1, tag, tag2, NULL);
  if (scroll)
  {
    gtk_text_buffer_get_end_iter (buffer, &end);
    mark = gtk_text_buffer_create_mark (buffer, "_end", &end, FALSE);
    gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (textview), mark, 0.0,
                                  TRUE, 0.5, 0.5);
    gtk_text_buffer_delete_mark_by_name (buffer, "_end");
  }
}

gchar *fromutf(const gchar *in)
{
  gsize r, w;
  return (!in) ? NULL: g_convert(in,-1,"iso-8859-2","UTF-8",&r,&w,NULL);
}

gchar *utf(const gchar *in)
{
  gsize r, w;
  return (!in) ? NULL: g_convert(in,-1,"UTF-8","iso-8859-2",&r,&w,NULL);
}


const gchar *textview_get_text(GtkWidget *textview)
{
  GtkTextBuffer *buffer;
  GtkTextIter start, end;

  if (!textview)
    return NULL;
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
  if (!buffer)
    return NULL;
  gtk_text_buffer_get_start_iter (buffer, &start);
  gtk_text_buffer_get_end_iter (buffer, &end);
  return gtk_text_buffer_get_text (buffer, &start, &end, TRUE);
}

void info (const gchar *s1)
{
  gchar *s;
  GtkWidget *window;
  GtkWidget *label;

  s = utf (s1);
  window = create_window_info ();
  label = lookup_widget (window, "label_message");
  gtk_label_set_text (GTK_LABEL (label), s);
  gtk_widget_show (window);
  g_free (s);
}
