/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/






#include "window-chat.h"

#include "main.h"
#include "support.h"
#include "utils.h"
#include "lists.h"
#include "icons.h"
#include "interface.h"
#include <string.h>

//window chat and message

void window_chat_set_status (const gchar *jid, UserStatus status)
{
  Talk *t;

  t = list_talk_item_get (jid);
  if (t)
  {
    gtk_window_set_icon (GTK_WINDOW (t->window),
                      gtk_image_get_pixbuf(GTK_IMAGE(icons_tlen_get(status))));
  }
}

void window_message_button_ok_clicked (GtkWidget *window)
{
  GtkWidget *widget;
  gchar *id, *message;
  TlenMessageType type;

  widget = lookup_widget (window, "entry_id");
  id = (gchar*) gtk_entry_get_text (GTK_ENTRY (widget));
  if (!tlen_strlen (id))
  {
    info ("Musisz poda identyfikator odbiorcy");
    return;
  }
  widget = lookup_widget (window, "textview");
  message = fromutf(textview_get_text (widget));
  if (!tlen_strlen (message))
  {
    info ("Pusta wiadomo");
    return;
  }
  if (!strchr (id, '@'))
    id = g_strdup_printf ("%s@tlen.pl", id);
  else
    id = g_strdup (id);
  widget = lookup_widget (window, "entry_type");
  if (!tlen_strcmp (gtk_entry_get_text (GTK_ENTRY(widget)), "Message"))
    type = TLEN_MESSAGE_TYPE_NORMAL;
  else
    type = TLEN_MESSAGE_TYPE_CHAT;
  if (!tlen_message (profile->tc, id, message, type))
    info ("Wystapi bd podczas wysyania wiadomoci");
  else
    gtk_widget_destroy (window);
  g_free (message);
  g_free (id);
}

void my_create_window_message (const gchar *jid, const gchar *message)
{
  GtkWidget *window, *entry;

  window = create_window_message ();
  if (jid)
  {
    entry = lookup_widget (window, "entry_id");
    gtk_entry_set_text (GTK_ENTRY (entry), jid);
    entry = lookup_widget (window, "textview");
    gtk_widget_grab_focus (entry);
  }
  gtk_widget_show (window);
}

void window_chat_close (GtkWidget *window)
{
  gtk_widget_hide (window);
}

void window_chat_button_send_clicked (GtkWidget *window)
{
  GtkWidget *textview;
  gchar *s;
  Talk *t;

  t = list_talk_item_get_by_window (window);
  if (!t)// that should not happened
  {
    tlen_debug ("Error, no window found\n");
    return;
  }
  textview = lookup_widget (window, "textview_bottom");
  s = fromutf (textview_get_text (textview));
  tlen_message (profile->tc, t->jid, s, TLEN_MESSAGE_TYPE_CHAT);
  textview_clear (textview);
  textview = lookup_widget (window, "textview_top");
  textview_append_message (textview, profile->name, s, NULL, FALSE);
  g_free (s);
}

void window_chat_append (const gchar *jid, const gchar *body,
                         const gchar *stamp, gboolean from)
{
  GtkWidget *window, *widget;
  Talk *t;
  GtkTextBuffer *buffer;
  gchar *user, *title;
  User *u;

  t = list_talk_item_get (jid);
  if (!t)
  {
    window = create_window_chat ();
    widget = lookup_widget (window, "textview_top");
    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
    gtk_text_buffer_create_tag (buffer, "fontbold", "font",
                                "bold", NULL);
    gtk_text_buffer_create_tag (buffer, "datefg", "foreground",
                                datefg, NULL);
    gtk_text_buffer_create_tag (buffer, "nickfg", "foreground",
                                nickfg, NULL);
//    gtk_text_buffer_create_tag (buffer, "textfg", "foreground",
//                                textfg, NULL);
    t = list_talk_item_new (window, jid);
    list_talk_item_add (t);
  }
  else
  {
    window = t->window;
  }
  gtk_widget_show (window);
  if (body)
  {
    widget = lookup_widget (window, "textview_top");
    textview_append_message (widget, jid, body, stamp, from);
  }
  user = list_user_get_name (jid);
  title = g_strdup_printf("Rozmowa: %s", user);
  gtk_window_set_title (GTK_WINDOW(window), title);
  u = list_user_item_get (jid);
  if (u)
    window_chat_set_status (jid, u->status);
  else
    window_chat_set_status (jid, USER_STATUS_UNAVAILABLE);
  g_free(user);
  g_free(title);
}

