/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/




// routines for window_login, window_profiles and window_profile

#include "window-login.h"
#include "window-misc.h"
#include "window-main.h"

#include "main.h"
#include "support.h"
#include "interface.h"
#include "utils.h"
#include "conf.h"
#include "support.h"
#include "tlen.h"

static
void _window_profiles_add_columns (GtkTreeView *treeview);
static
GtkTreeModel *_window_profiles_create_model();
static void _window_login_load_combo ();
void _window_profiles_delete (gpointer d1, gpointer d2);

static Profile *change_profile;

void window_login_close (void)
{
  if (window_login)
  {
    gtk_widget_destroy (window_login);
    window_login = NULL;
  }
  main_quit ();
}

void my_create_window_login (void)
{
  if (window_login)
    return;
  window_login = create_window_login ();
  _window_login_load_combo ();
  gtk_widget_show (window_login);
}

void window_login_button_profiles_clicked (void)
{
  gtk_widget_destroy (window_login);
  window_login = NULL;
  my_create_window_profiles ();
}

void window_login_button_open_clicked (void)
{
  GtkWidget *entry;
  const gchar *name;
  gchar *pname;
  Profile *p;

  entry = lookup_widget (window_login, "combo_entry");
  name = gtk_entry_get_text (GTK_ENTRY(entry));
  pname = fromutf(name);
  p = list_profile_item_get (pname);
  g_free (pname);
  if(!p)
  {
    info ("Kliknij przycisk Profile aby doda jaki profil.");
    return;
  }
  gtk_widget_destroy (window_login);
  window_login = NULL;
  my_create_window_main (p);
  tc_open (p);
}

void my_create_window_profiles (void)
{
  GtkWidget *treeview;
  GtkTreeModel *model;

  if (window_profiles)
    return;
  window_profiles = create_window_profiles ();
  treeview = lookup_widget (window_profiles, "treeview");
  _window_profiles_add_columns (GTK_TREE_VIEW (treeview));
  model = _window_profiles_create_model();
  gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
  g_object_unref (G_OBJECT (model));
  gtk_widget_show (window_profiles);
}

static
void _window_profiles_add_columns (GtkTreeView *treeview)
{
  GtkCellRenderer *text;
  GtkTreeViewColumn *column;

  text = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes (NULL, text,
                                                     "text",0,
                                                     NULL);
  gtk_tree_view_append_column (treeview, column);
//  text = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes (NULL, text,
                                                     "text",1,
                                                     NULL);
  gtk_tree_view_append_column (treeview, column);
}

static
GtkTreeModel *_window_profiles_create_model()
{
  GtkTreeStore *model;
  GtkTreeIter iter;
  GSList *l = list_profiles;
  Profile *p;
  gchar *s, *w;

  model = gtk_tree_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
  for(;l;l=l->next)
  {
    p = (Profile*) l->data;
    gtk_tree_store_append (model, &iter, NULL);
    if (p->active)
      w = utf (" (domylny)");
    else
      w = g_strdup ("");
    s = utf (p->name);
    gtk_tree_store_set (model, &iter,
                        0, s,
                        1, w,
                        -1);
    g_free (w);
    g_free (s);
  }
  return GTK_TREE_MODEL(model);
}

void window_profiles_close (void)
{
  if (window_profiles)
  {
    gtk_widget_destroy (window_profiles);
    window_profiles = NULL;
  }
  conf_save_profiles ();
//  if (state == STATE_PROFILES)
  my_create_window_login ();
}

void window_profiles_button_new_clicked (void)
{
  gtk_widget_destroy (window_profiles);
  window_profiles = NULL;
  my_create_window_profile (NULL);
}

void window_profiles_button_delete_clicked (void)
{
  my_create_window_yesno ("Skasowa wybrany profil?", _window_profiles_delete,
                          NULL, NULL, NULL);
}

void window_profiles_button_edit_clicked (void)
{
  GtkWidget *treeview;
  GtkTreeSelection *selection;
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *pname, *s;
  Profile *p;

  treeview = lookup_widget (window_profiles, "treeview");
  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gtk_tree_model_get (model, &iter,
                        0, &pname,
                        -1);
    s = fromutf (pname);
    p = list_profile_item_get (s);
    gtk_widget_destroy (window_profiles);
    window_profiles = NULL;
    my_create_window_profile (p);
    g_free (s);
  }
}

void _window_profiles_delete (gpointer d1, gpointer d2)
{
  GtkWidget *treeview;
  GtkTreeSelection *selection;
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *pname, *s;
  Profile *p;

  treeview = lookup_widget (window_profiles, "treeview");
  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gtk_tree_model_get (model, &iter,
                        0, &pname,
                        -1);
    s = fromutf (pname);
    p = list_profile_item_get (s);
    list_profile_item_del (p);
    g_free (s);
    model = _window_profiles_create_model ();
    gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
    g_object_unref (G_OBJECT (model));
  }
}

void my_create_window_profile (Profile *p)
{
  GtkWidget *button;
  gchar *u;

  if (window_profile)
    return;
  change_profile = p;
  window_profile = create_window_profile ();
  if (p)
  {
    button = lookup_widget (window_profile, "checkbutton_active");
    if (p->active)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
    else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
    button = lookup_widget (window_profile, "entry_name");
//    gtk_widget_set_sensitive (button, FALSE);
    u = utf (p->name);
    gtk_entry_set_text (GTK_ENTRY (button), u);
    g_free (u);
    button = lookup_widget (window_profile, "entry_tlen_login");
    u = utf (p->tlen_login);
    gtk_entry_set_text (GTK_ENTRY (button), u);
    g_free (u);
    button = lookup_widget (window_profile, "entry_tlen_password");
    u = utf (p->tlen_password);
    gtk_entry_set_text (GTK_ENTRY (button), u);
    g_free (u);
  }
  gtk_widget_show (window_profile);
}

void window_profile_close (void)
{
  if (window_profile)
  {
    gtk_widget_destroy (window_profile);
    window_profile = NULL;
  }
  my_create_window_profiles ();
}

void window_profile_button_ok_clicked (void)
{
  GtkWidget *entry, *button;
  gchar *name, *tlen_login, *tlen_password;
  Profile *p;
  gboolean active;

  entry = lookup_widget (window_profile, "entry_name");
  name = fromutf (gtk_entry_get_text (GTK_ENTRY (entry)));
  entry = lookup_widget (window_profile, "entry_tlen_login");
  tlen_login = fromutf (gtk_entry_get_text (GTK_ENTRY (entry)));
  entry = lookup_widget (window_profile, "entry_tlen_password");
  tlen_password = fromutf (gtk_entry_get_text (GTK_ENTRY (entry)));
  if (!name || !tlen_strlen (name) ||
      !tlen_login || !tlen_strlen (tlen_login) ||
      !tlen_password || !tlen_strlen (tlen_password))
  {
    g_free (name);
    g_free (tlen_login);
    g_free (tlen_password);
    info ("Bd we wprowadzonych danych");
    return;
  }
  button = lookup_widget (window_profile, "checkbutton_active");
  active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
  p = list_profile_item_get (name);
  if (p && (!change_profile || (change_profile && (change_profile != p))))
  {
    info ("Profil o podanej nazwie ju istnieje");
    g_free (name);
    g_free (tlen_login);
    g_free (tlen_password);
    return;
//    g_free (p->tlen_login);
//    g_free (p->tlen_password);
//    p->tlen_login = tlen_login;
//    p->tlen_password = tlen_password;
  }
  else
  {
    p = list_profile_item_new ();
    p->name = name;
    p->tlen_login = tlen_login;
    p->tlen_password = tlen_password;
    p->active = active;
    if (change_profile && (change_profile != p))
      list_profile_item_del (change_profile);
    list_profile_item_add (p);
  }
  window_profile_close ();
  my_create_window_profiles ();
}

static void _window_login_load_combo ()
{
  GList *l = NULL;
  GSList *l2 = list_profiles;
  Profile *p;
  GtkWidget *widget;
  gchar *s;

  if (l2)
  while (l2)
  {
    p = (Profile*) l2->data;
    if (p)
    {
      s = utf (p->name);
      if (s)
        l = g_list_append (l, (gpointer) s);
    }
    l2 = l2->next;
  }
  else
  {
    s = g_strdup ("");
    l = g_list_append (l, (gpointer) s);
  }
  widget = lookup_widget (window_login, "combo1");
  gtk_combo_set_popdown_strings (GTK_COMBO(widget), l);
  g_list_foreach (l, frees, NULL);
  g_list_free (l);
}

