/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/



#include "window-main.h"
#include "window-chat.h"
#include "interface.h"
#include "main.h"
#include "support.h"
#include "lists.h"
#include "icons.h"
#include "utils.h"
#include "menu.h"
#include <gdk/gdkkeysyms.h>

GtkTooltips *status_tooltips; //tooltips list for status icons


static void
_on_treeview_main_row_activated             (GtkTreeView     *treeview,
                                             GtkTreePath     *arg1,
                                             GtkTreeViewColumn *arg2,
                                             gpointer         user_data);
static void
_on_treeview_main_button_release_event      (GtkWidget       *widget,
                                             GdkEventButton  *event,
                                             gpointer         user_data);

static void _treeview_func_text (GtkTreeViewColumn *tree_column,
                                 GtkCellRenderer   *cell,
                                 GtkTreeModel      *model,
                                 GtkTreeIter       *iter,
                                 gpointer           data);
static
void _treeview_func_pixbuf (GtkTreeViewColumn *tree_column,
                            GtkCellRenderer   *cell,
                            GtkTreeModel      *model,
                            GtkTreeIter       *iter,
                            gpointer           data);
static
void _treeview_add_columns (GtkTreeView *treeview);
static
GtkTreeModel * _treeview_create_model ();



void window_main_set_status (UserStatus status)
{
  if (window_main)
  {
    status_button_set (status);
    gtk_window_set_icon(GTK_WINDOW(window_main),
                        gtk_image_get_pixbuf(GTK_IMAGE(
                                                  icons_tlen_get (status))));
  }
}

void my_create_window_main (Profile *p)
{
  GtkWidget *widget, *button;
  GtkWidget *menu_status;
  GtkWidget *image;
  gchar *s;
  GtkWidget *menuitem;
  GtkWidget *menu_tlen;
  GtkAccelGroup *accel_group;


  accel_group = gtk_accel_group_new ();

  if (window_main)
    return;
  window_main = create_window_main ();

  status_tooltips = gtk_tooltips_new(); //init. tooltips list for status icons


  //main menu
  menu_tlen = gtk_menu_new ();
  GLADE_HOOKUP_OBJECT(window_main, menu_tlen, "menu_tlen");
  button=lookup_widget(window_main, "button_main_menu");
  g_signal_connect_swapped (G_OBJECT (button), "event",
                            G_CALLBACK (menu_show),
                            menu_tlen);

  s = utf("Wy_lij wiadomo...");
  menuitem = gtk_image_menu_item_new_with_mnemonic (s);
  g_free(s);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  image = create_pixmap (window_main, "sendmsg.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "sendmsg.png");
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                      G_CALLBACK (on_wyslij_wiadomosc_activate),
                      NULL);

  s = utf("Wy_lij plik...");
  menuitem = gtk_image_menu_item_new_with_mnemonic (s);
  g_free(s);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  image = create_pixmap (window_main, "menu-sendfile.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "menu-sendfile.png");
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                      G_CALLBACK (on_wyslij_plik_activate),
                      NULL);
  gtk_widget_set_sensitive (menuitem, FALSE);

  s = utf("Mo_ja wizytwka...");
  menuitem = gtk_image_menu_item_new_with_mnemonic (s);
  g_free(s);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  image = create_pixmap (window_main, "wizytowka.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "wizytowka.png");
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                      G_CALLBACK (on_wizytowka_activate),
                      NULL);

  menuitem = gtk_image_menu_item_new_with_mnemonic ("Wy_szukiwanie...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_S, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  image = gtk_image_new_from_stock ("gtk-find", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                    G_CALLBACK (on_wyszukiwanie_activate),
                    NULL);

  menuitem = gtk_image_menu_item_new_with_mnemonic ("_Dodaj kontakt...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_A, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  image = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                    G_CALLBACK (on_dodaj_kontakt_activate),
                    NULL);

  menuitem = gtk_menu_item_new ();
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  gtk_widget_set_sensitive (menuitem, FALSE);

  menuitem = gtk_image_menu_item_new_with_mnemonic ("_Ustawienia...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_P, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  image = gtk_image_new_from_stock ("gtk-preferences", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
  gtk_widget_set_sensitive (menuitem, FALSE);

  menuitem = gtk_menu_item_new ();
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  gtk_widget_set_sensitive (menuitem, FALSE);

  menuitem = gtk_image_menu_item_new_with_mnemonic ("_O programie...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  image = gtk_image_new_from_stock ("gtk-dialog-info", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                    G_CALLBACK (on_o_programie_activate),
                    NULL);

  menuitem = gtk_menu_item_new ();
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  gtk_widget_set_sensitive (menuitem, FALSE);

  menuitem = gtk_image_menu_item_new_with_mnemonic ("_Wyloguj profil...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_W, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                    G_CALLBACK (on_wyloguj_profil_activate),
                    NULL);

  image = gtk_image_new_from_stock ("gtk-jump-to", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);

  menuitem = gtk_image_menu_item_new_with_mnemonic ("W_yj\233cie");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_tlen), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_Q, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  image = gtk_image_new_from_stock ("gtk-quit", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                    G_CALLBACK (on_wyjscie_activate),
                    NULL);



  //menu status
  menu_status = gtk_menu_new ();
  GLADE_HOOKUP_OBJECT (window_main, menu_status, "menu_status");
  button = lookup_widget (window_main, "button_main_status");
  g_signal_connect_swapped (G_OBJECT (button), "event",
                            G_CALLBACK (menu_show),
                            menu_status);

  image = create_pixmap (window_main, "online.png");
  s = utf ("_Dostpny");
  menuitem = gtk_image_menu_item_new_with_mnemonic (s);
  g_free (s);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_status), menuitem);
  g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
                             G_CALLBACK (menu_status_response),
                             "1");
  gtk_widget_show (menuitem);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
  gtk_widget_show (image);
  image = create_pixmap (window_main, "online.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "im_online");

  image = create_pixmap (window_main, "chat.png");
  s = utf("_Porozmawiajmy");
  menuitem = gtk_image_menu_item_new_with_mnemonic (s);
  g_free (s);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_status), menuitem);
  g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
                             G_CALLBACK (menu_status_response),
                             "2");
  gtk_widget_show (menuitem);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
  gtk_widget_show (image);
  image = create_pixmap (window_main, "chat.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "im_chat");

  image = create_pixmap (window_main, "occupied.png");
  s=utf("_Jestem zajty");
  menuitem=gtk_image_menu_item_new_with_mnemonic(s);
  g_free(s);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_status),menuitem);
  g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
                             G_CALLBACK (menu_status_response),
                             "3");
  gtk_widget_show (menuitem);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),image);
  gtk_widget_show (image);
  image = create_pixmap (window_main, "occupied.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "im_occupied");

  image = create_pixmap (window_main, "away.png");
  s=utf("_Zaraz wracam");
  menuitem=gtk_image_menu_item_new_with_mnemonic(s);
  g_free(s);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_status),menuitem);
  g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
                             G_CALLBACK (menu_status_response),
                             "4");
  gtk_widget_show (menuitem);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),image);
  gtk_widget_show (image);
  image = create_pixmap (window_main, "away.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "im_away");

  image = create_pixmap (window_main, "beback.png");
  s=utf("_Wrc pniej");
  menuitem=gtk_image_menu_item_new_with_mnemonic(s);
  g_free(s);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_status),menuitem);
  g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
                             G_CALLBACK (menu_status_response),
                             "5");
  gtk_widget_show (menuitem);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),image);
  gtk_widget_show (image);
  image = create_pixmap (window_main, "beback.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "im_beback");

  image = create_pixmap (window_main, "invisible.png");
  s=utf("N_iewidoczny");
  menuitem=gtk_image_menu_item_new_with_mnemonic(s);
  g_free(s);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_status),menuitem);
  g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
                             G_CALLBACK (menu_status_response),
                             "6");
  gtk_widget_show (menuitem);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),image);
  gtk_widget_show (image);
  image = create_pixmap (window_main, "invisible.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "im_invisible");

  image = create_pixmap (window_main, "offline.png");
  s=utf("_Niedostpny");
  menuitem=gtk_image_menu_item_new_with_mnemonic(s);
  g_free(s);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_status),menuitem);
  g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
                             G_CALLBACK (menu_status_response),
                             "7");
  gtk_widget_show (menuitem);
  gtk_widget_set_sensitive (menuitem, FALSE);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),image);
  gtk_widget_show (image);
  image = create_pixmap (window_main, "offline.png");
  gtk_widget_show (image);
  GLADE_HOOKUP_OBJECT (window_main, image, "im_offline");

  menuitem = gtk_menu_item_new ();
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_status), menuitem);
  gtk_widget_set_sensitive (menuitem, FALSE);
  s=utf("Wasny opis...");
  menuitem = gtk_image_menu_item_new_with_label (s);
  g_free(s);
  gtk_widget_show (menuitem);
  gtk_widget_set_sensitive (menuitem, FALSE);
  gtk_container_add (GTK_CONTAINER (menu_status), menuitem);
  g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
                             G_CALLBACK (menu_status_response),
                             "8");
  image = create_pixmap (window_main, "info.png");
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem),image);
  gtk_widget_show (image);


  profile = p;
  widget = lookup_widget (window_main, "treeview_main");
  if (widget)
  {
    _treeview_add_columns (GTK_TREE_VIEW (widget));
    g_signal_connect (G_OBJECT (widget), "row_activated",
                      G_CALLBACK (_on_treeview_main_row_activated),
                      NULL);
    g_signal_connect (G_OBJECT (widget), "button_release_event",
                      G_CALLBACK (_on_treeview_main_button_release_event),
                      NULL);
  }
  icons_tlen_create ();
  gtk_window_add_accel_group (GTK_WINDOW (window_main), accel_group);
  gtk_widget_show (window_main);
}

void window_main_close ()
{
  if (window_main)
  {
    gtk_widget_destroy (window_main);
    window_main = NULL;
    list_user_free ();
  }
}

void window_main_treeview_update()
{
  GtkWidget *treeview;
  GtkTreeModel *model;

  if (!window_main)
    return;
  treeview = lookup_widget (window_main, "treeview_main");
  if (!treeview)
    return;
  list_user_sort ();
  model = _treeview_create_model ();
  gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
//  if(preferences & PREF_GROUPS_LIST)
//    reload_expanded(treeview);
  g_object_unref (G_OBJECT (model));
}

static
void _treeview_add_columns (GtkTreeView *treeview)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  renderer = gtk_cell_renderer_pixbuf_new();
  column = gtk_tree_view_column_new();
  gtk_tree_view_column_pack_start(column, renderer, FALSE);
  gtk_tree_view_column_set_cell_data_func (column, renderer,
                                           _treeview_func_pixbuf,
                                           NULL, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_column_pack_start(column, renderer, FALSE);
  gtk_tree_view_column_set_cell_data_func (column, renderer,
                                           _treeview_func_text,
                                           NULL, NULL);
  gtk_tree_view_append_column (treeview, column);

}


static
void _treeview_func_pixbuf (GtkTreeViewColumn *tree_column,
                            GtkCellRenderer   *cell,
                            GtkTreeModel      *model,
                            GtkTreeIter       *iter,
                            gpointer           data)
{
  UserStatus status;
  gchar *jid;
  User *u;
  guint row_type;

  gtk_tree_model_get (model, iter,
                      0, &status,
                      1, &jid,
                      3, &row_type,
                      -1);
  if (row_type != ROW_TYPE_USER)
    g_object_set (GTK_CELL_RENDERER (cell), "visible", FALSE, NULL);
  else
  {
    g_object_set (GTK_CELL_RENDERER (cell), "visible", TRUE, NULL);
    u = list_user_item_get (jid);
    if (!u)
      return;
    switch (u->type)
    {
      case USER_TYPE_TLEN:
        g_object_set (GTK_CELL_RENDERER (cell),
                      "pixbuf", gtk_image_get_pixbuf(
                                 GTK_IMAGE(icons_tlen_get (status))),
                      NULL);
        break;
    }
  }

}

static void _treeview_func_text (GtkTreeViewColumn *tree_column,
                                 GtkCellRenderer   *cell,
                                 GtkTreeModel      *model,
                                 GtkTreeIter       *iter,
                                 gpointer           data)
{
  UserStatus status;
  gchar *name;
  guint row_type;

  gtk_tree_model_get (model, iter,
                      0, &status,
                      2, &name,
                      3, &row_type,
                      -1);
  switch (row_type)
  {
    case ROW_TYPE_USER:
      g_object_set (GTK_CELL_RENDERER (cell),
                    "font", "normal",
                    "foreground", "black",
                    "scale", 1.0,
                    NULL);
      g_object_set (GTK_CELL_RENDERER (cell),
                    "text", NULL,
                    "markup", name,
                    NULL);
      return;
      break;
    case ROW_TYPE_LABEL:
      g_object_set (GTK_CELL_RENDERER (cell),
                    "font", "bold",
                    "foreground", "darkgrey",
                    "scale", 1.0,
                    "text", name,
                    NULL);
      break;
    case ROW_TYPE_GROUP:
      g_object_set (GTK_CELL_RENDERER (cell),
                    "font", "bold",
                    "foreground", "blue",
                    "scale", 1.0,
                    NULL);
      g_object_set (GTK_CELL_RENDERER (cell),
                    "markup", NULL,
                    "text", name,
                    NULL);
      break;
  }
}

static
GtkTreeModel * _treeview_create_model ()
{
  GtkTreeStore *store;
  GtkTreeIter child_iter, *iter = NULL;
  GSList *l;
  User *u;
  gchar *s, *ut, *desc, *escaped;
  gboolean b;

  store = gtk_tree_store_new (4, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING,
                              G_TYPE_UINT);
  // full subscription
  l = list_users;
  while(l)
  {
    u = (User*) l->data;
    if (user_subscribed (u))
    {
      gtk_tree_store_append (store, &child_iter, iter);
      s = list_user_get_name (u->jid);
      ut = utf (s);
      g_free (s);
      s = g_markup_escape_text (ut, tlen_strlen(ut));
      g_free (ut);
      desc = user_description (u);
      if (desc)
      {
        escaped = g_markup_escape_text (desc, tlen_strlen(desc));
        g_free (desc);
        ut = g_strdup_printf ("%s\n<span size='smaller'>%s</span>", s,
                              escaped);
        g_free (escaped);
      }
      else
        ut = g_strdup (s);
      g_free (s);
      gtk_tree_store_set (store, &child_iter,
                          0, u->status,
                          1, u->jid,
                          2, ut,
                          3, ROW_TYPE_USER,
                          -1);
      g_free (ut);
    }
    l = l->next;
  }
  l = list_users;
  b = TRUE;
  while(l)
  {
    u = (User*) l->data;
    if (u && !tlen_strcmp (u->ask, "subscribe"))
    {
      if (b)
      {
        gtk_tree_store_append (store, &child_iter, iter);
        gtk_tree_store_set (store, &child_iter,
                            2, "Oczekiwanie",
                            3, ROW_TYPE_LABEL,
                            -1);
        b = FALSE;
      }
      gtk_tree_store_append (store, &child_iter, iter);
      s = list_user_get_name (u->jid);
      ut = utf (s);
      g_free (s);
      gtk_tree_store_set (store, &child_iter,
                          0, u->status,
                          1, u->jid,
                          2, ut,
                          3, ROW_TYPE_USER,
                          -1);
      g_free (ut);
    }
    l = l->next;
  }
  l = list_users;
  b = TRUE;
  while(l)
  {
    u = (User*) l->data;
    if (u && !user_subscribed (u) && !tlen_strlen (u->ask))
    {
      if (b)
      {
        gtk_tree_store_append (store, &child_iter, iter);
        gtk_tree_store_set (store, &child_iter,
                            2, "Brak autoryzacji",
                            3, ROW_TYPE_LABEL,
                            -1);
        b = FALSE;
      }
      gtk_tree_store_append (store, &child_iter, iter);
      s = list_user_get_name (u->jid);
      ut = utf (s);
      g_free (s);
      gtk_tree_store_set (store, &child_iter,
                          0, u->status,
                          1, u->jid,
                          2, ut,
                          3, ROW_TYPE_USER,
                          -1);
      g_free (ut);
    }
    l = l->next;
  }

  return GTK_TREE_MODEL (store);
}

static void
_on_treeview_main_button_release_event      (GtkWidget       *widget,
                                             GdkEventButton  *event,
                                             gpointer         user_data)
{
  menu_user_open (widget, event);
}

static void
_on_treeview_main_row_activated             (GtkTreeView     *treeview,
                                             GtkTreePath     *arg1,
                                             GtkTreeViewColumn *arg2,
                                             gpointer         user_data)
{
  GtkTreeSelection *selection;
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *jid;
  guint row_type;

  selection = gtk_tree_view_get_selection (treeview);
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gtk_tree_model_get (model, &iter,
                        1, &jid,
                        3, &row_type,
                        -1);
    if (row_type == ROW_TYPE_USER)
      window_chat_append (jid, NULL, NULL, FALSE);
  }

}


