/*
** TleenX3 (Tlen.pl Client)
** Copyright (c) 2004 Hubert Sokoowski <who_ami@tlen.pl>
**                    Pawe Biliski <rael@fr.pl>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
*/




#include "window-misc.h"
#include "interface.h"
#include "main.h"
#include "support.h"
#include "utils.h"
#include <string.h>


static YesNoFunc _funcok, _funcno;
static gpointer _d1, _d2;


void my_create_window_webmsg (const gchar *from, const gchar *email,
                              const gchar *body)
{
  GtkWidget *window, *label;
  gchar *f, *e, *s, *b;

  window = create_window_webmsg ();
  f = utf (from);
  e = utf (email);
  b = utf (body);
  if (e)
    s = g_strdup_printf ("%s (%s)", f, e);
  else
    s = g_strdup (from);
  g_free (f);
  g_free (e);
  label = lookup_widget (window, "label_from");
  gtk_label_set_text (GTK_LABEL (label), s);
  label = lookup_widget (window, "textview");
  textview_append (label, b, 0, NULL, NULL);
  g_free (s);
  g_free (b);
  gtk_widget_show (window);
}

void my_create_window_newmail (const gchar *from, const gchar *subject)
{
  GtkWidget *window, *label;
  gchar *f, *s;

  window = create_window_newmail ();
  f = utf (from);
  s = utf (subject);
  label = lookup_widget (window, "label_from");
  gtk_label_set_text (GTK_LABEL (label), f);
  label = lookup_widget (window, "label_subject");
  gtk_label_set_text (GTK_LABEL (label), s);
  g_free (f);
  g_free (s);
  gtk_widget_show (window);
}

void window_contact_ok_clicked (GtkWidget *window)
{
  GtkWidget *entry;
  gchar *name, *id, *group, *uname, *ugroup;
  User *u;
  gchar *login;
  gboolean ret;

  entry = lookup_widget (window, "entry_id");
  id = (gchar*) gtk_entry_get_text (GTK_ENTRY (entry));
  if (tlen_strlen (id) == 0)
  {
    info ("Musisz poda identyfikator");
    return;
  }
  entry = lookup_widget (window, "entry_name");
  name = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
  if (!tlen_strlen (name))
    name = NULL;
  entry = lookup_widget (window, "entry_group");
  group = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
  if (!tlen_strlen (group))
    group = NULL;
  uname = fromutf (name);
  ugroup = fromutf (group);
  ret = tlen_contact_add (profile->tc, id, uname, ugroup);
  g_free (uname);
  g_free (ugroup);
  if (!ret)
  {
    info ("Wystpi bd podczas dodawania kontaktu");
    return;
  }
  if (strchr (id, '@'))
    login = g_strdup (id);
  else
    login = g_strdup_printf ("%s@tlen.pl", id);
  u = list_user_item_get (login);
  if (!user_subscribed (u))
  {
    ret = tlen_subscribe_request (profile->tc, id);
    if (!ret)
      info ("Wystpi bd podczas dodawania kontaktu");
  }
  window_contact_close (window);
}

void window_contact_close (GtkWidget *window)
{
  gtk_widget_destroy (window);
}

void my_create_window_contact (const gchar *id, gint type)
{
  GtkWidget *entry, *window;
  User *u = NULL;
  gchar *s;

  window = create_window_contact ();
  entry = lookup_widget (window, "entry_id");
  if (id)
  {
    s = NULL;
    gtk_entry_set_text (GTK_ENTRY (entry), id);
    gtk_widget_set_sensitive (entry, FALSE);
    u = list_user_item_get (id);
  }
  if(u)
  {
    gtk_window_set_title (GTK_WINDOW (window), "Edycja danych");
    if (u->name)
    {
      entry = lookup_widget (window, "entry_name");
      s = utf (u->name);
      gtk_entry_set_text (GTK_ENTRY (entry), s);
      g_free (s);
    }
    if (u->group)
    {
      entry = lookup_widget (window, "entry_group");
      s = utf (u->group);
      gtk_entry_set_text (GTK_ENTRY (entry), s);
      g_free (s);
    }
  }
  gtk_widget_show(window);
}

void my_create_window_yesno (const gchar *question, YesNoFunc funcok,
                             YesNoFunc funcno,
                             gpointer d1, gpointer d2)
{
  GtkWidget *label;
  gchar *u;

  window_yesno = create_window_yesno ();
  label = lookup_widget (window_yesno, "label_question");
  u = utf (question);
  gtk_label_set_text (GTK_LABEL (label), u);
  g_free (u);
  _funcok = funcok;
  _funcno = funcno;
  _d1 = d1;
  _d2 = d2;
  gtk_widget_show (window_yesno);
}

void window_yesno_close ()
{
  if (window_yesno)
  {
    gtk_widget_destroy (window_yesno);
    window_yesno = NULL;
    if (_funcno)
    {
      _funcno (_d1, _d2);
      _funcno = NULL;
    }
  }
}

void window_yesno_button_cancel_clicked ()
{
  window_yesno_close ();
}

void window_yesno_button_ok_clicked ()
{
  gtk_widget_destroy (window_yesno);
  window_yesno = NULL;
  if (_funcok)
  {
    _funcok (_d1, _d2);
    _funcok = NULL;
  }
}
